*+--------------------------------------------------------------------
*+
*+ Source Module => c:\hmg.3.4.4\0\BACKUP\\REGCLASS.PRG
*+
*+    Copyright(C) 1983-2021 by Auge & Ohr
*+
*+    Functions: Class XbpReg
*+               Method XbpReg:Init()
*+               Method XbpReg:Create()
*+               Method XbpReg:RegOpenKeyEx()
*+               Method XbpReg:RegCloseKey()
*+               Method XbpReg:RegCreateKeyEx()
*+               Method XbpReg:NewKey()
*+               Method XbpReg:DelKey()
*+               Method XbpReg:RegQueryInfoKey()
*+               Method xbpReg:RegEnumKeyEx()
*+               Method xbpReg:RegEnumValue()
*+               Method XbpReg:GetValue()
*+               Method XbpReg:SetValue()
*+               Method XbpReg:DelValue()
*+               Method XbpReg:Standard()
*+               Method XbpReg:ReadBinType()
*+               Method XbpReg:SaveStringType()
*+               Method XbpReg:Status()
*+               Method XbpReg:KeyName()
*+               Method XbpReg:KeyList()
*+               Method XbpReg:ValueList()
*+               Method xbpReg:MoveUp()
*+               Method xbpReg:MoveDown()
*+               Static Function ATNUM()
*+
*+    Reformatted by Click! 2.05.37 on Jul-12-2021 at  7:08 am
*+
*+--------------------------------------------------------------------

/*
   Unicode Version for harbour HMG / Extended Version
   2021 by Jimmy, Auge_Ohr@Web.DE

   change for HMG

   STATIC UseAnsi := .T. / .F.

   METHOD Standard ACCESS ASSIGN
   METHOD Status   ACCESS
   METHOD KeyName  ACCESS

   ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   DllUnload( ::nDllHandle )


*************************** Original ***************************


   Registry-Klassenbibliothek fr Xbase++
=======================================================
   Registry class for Xbase++

(C) Thomas Braun Softwareentwicklung
    Hindenburgstr. 69
    D-72336 Balingen
    GERMANY

    e-mail   : tb@software-braun.de
    Homepage : http://www.software-braun.de

Feel free to use the source below in any project you
like as long as you keep the above copyright note.

The most current version of this class can be found on
my homepage.

Any comments and enhancements are always welcome.

If you have any questions regarding this source feel free
to contact me via e-mail.

---------------------------------------------------------------------------------------
CHANGE HISTORY (sorry german guys, only in english ;-) :

30.09.2001 Extended RegOpenKeyEx to support Windows NT / 2000 access right settings
           for registry keys. Versions before this change allways tried to open the
           registry key with maximum access rights which is not possible under NT
           for "regular" users but only for users with administrator privilege.

23.09.2001 Two new defines and some helpful functions from Scott McNay (see reghelp.prg)

02.10.1999 New iVar cSaveStringTypeAs
           New ACCESS/ASSIGN SaveStringType

29.08.1999 Fixed a weird "duplicate variable declaration" error with Xbase++ SL2 1.20.204

21.08.1999 removed a left over "Return lSuccess" in xbpReg:DelKey()
           Fixed bug in :Init() method with SET EXACT ON. Init is now saving
           the current EXACT-state and restoring it when finished. (Thanks to
           Ken Levitt for this)

18.04.1999 Added Section "CHANGE HISTORY"
           New iVar cReadBinaryTypeAs
           New ACCESS/ASSIGN ReadBinType
           New Method ::ValueList() for retrieving a 2-dim array that contains all
                                    value names - value pairs. (so now you can query if
                                    a value already exists)
           several minor bugfixes regarding the useage under WindowsNT
           (thanks to Axel Zimelka for this one and Johan Droskie for some
            other comments)

*/

#include "common.ch"
#include "dll.ch"
#include "regclass.ch"
#include "set.ch"

#IFDEF __XPP__
STATIC UseAnsi := .T.

#ELSE
#include "hbclass.ch"
//    #include "hbxbp.ch"
#xtranslate DllCall              => HMG_CallDLL
#xtranslate Var2Bin              => HB_Serialize
#xtranslate Bin2Var              => HB_Deserialize
STATIC UseAnsi := .F.

/**************************************************
FUNCTION DllLoad(cDLL)
LOCAL nDLL := HMG_CallDLL( "Kernel32.dll",, "LoadLibrary", "ADVAPI32.DLL" )
RETURN nDLL

FUNCTION DllUnload(nDLL)
RETURN FreeLibrary( nDLL )
**************************************************/

#ENDIF

/*
 Registry Klasse, verwaltet Informationen und stellt Zugriffsfunktionen fr einen
 bestimmten RegistryKey bereit. Dieser mu in der Init-Funktion angegeben werden.

 Die Manipulationsmethoden erlauben es dann, Werte und weitere Keys hinzu-
 zufgen / zu lschen / abzufragen usw.
 =================================================================================
 Registry class, contains information and accessfunctions for a single
 registry key. The name of the key must be set with the init method.

 The manipulation methods then allow to add/delete keys and values

*/

*+--------------------------------------------------------------------
*+
*+    Class XbpReg
*+
*+--------------------------------------------------------------------
*+
CLASS XbpReg

   HIDDEN:
   METHOD RegOpenKeyEx, RegCreateKeyEx, RegQueryInfoKey, RegCloseKey
   METHOD RegEnumKeyEx, RegEnumValue
   VAR nHKey                                                          // numerical handle from regclass.ch for the root key
   VAR cKey                                                           // Name of Subkey to open
   VAR nKeyHandle                                                     // Handle to currently open key
   VAR lStatus                                                        // State of Keys (see comment in the init method)
   VAR nCallSuccess                                                   // return value of all Reg...DLL functions
   VAR nDllHandle                                                     // DLL handle
   VAR xValue                                                         // general purpose iVar
   VAR cClass
   VAR nClass
   VAR nSubKeys
   VAR nMaxSubKeyLen
   VAR nMaxClassLen
   VAR nValues
   VAR nMaxValueNameLen
   VAR nMaxValueLen
   VAR nSecurityDescriptor
   VAR cLastWriteTime
   VAR nLastWriteTime
   VAR cReadBinaryTypeAs                                              // Datatype to be retrieved from REG_BINARY registry entry
   VAR cSaveStringTypeAs                                              // Datatype to be saved to registry

   EXPORTED:
   METHOD INIT, CREATE
   METHOD NewKey, DelKey
   METHOD GetValue, SetValue, DelValue
   METHOD KeyList, MoveUp, MoveDown
   METHOD ValueList

   METHOD Standard                                                    // ACCESS ASSIGN
   METHOD ReadBinType                                                 // get/set ::cReadBinaryTypeAs
   METHOD SaveStringType                                              // get/set ::cSaveStringTypeAs
   METHOD Status                                                      // ACCESS
   METHOD KeyName                                                     // ACCESS

   ENDCLASS

   /*
   Init-Methode. Ermitteln, welcher Root-Key bentigt wird und diesen in ::nHKey
   eintragen.

   Subkey ermitteln und diesen in ::cKey eintragen

   Die Memberfunktion ::Status() gibt einen logischen Wert zurck der angibt,
   ob der Key bereits in der Registry angelegt ist (.T.) oder nicht (.F.)

   Wird lCreate auf .T. (default = .F.) gesetzt und existiert der Key noch nicht,
   so wird er neu angelegt. Anderenfalls mu er mittels der Methode ::create()
   erzeugt werden.

=======================================================================================

   Init method. First determine the needed root key which is then stored in ::nHKey

   Determine subkey name and store into ::cKey

   The member function ::Status() returns a logical value that states if the key already
   exists in the registry (.T.) or not (.F.)

   If lCreate is set to .T. (default = .F.) and the key does not exist, it is
   created in the init method.

   Otherwise it has to be created via the :create() method.

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:Init()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:Init( cRegKey, lCreate, lReInit )

LOCAL nPos := 0
   /*
      Turning EXACT off, otherwise some string compares will fail...
      ======================
      EXACT auf "OFF" schalten, weil sonst einige String-Vergleiche weiter
      unten scheitern...
   */
LOCAL lSetExact := SET( _SET_EXACT, .F. )

   DEFAULT cRegKey TO ""
   DEFAULT lCreate TO .F.
   DEFAULT lReInit TO .F.

   ::lStatus := .F.
   ::nDllHandle := 0
   ::cClass := ""
   ::nClass := 0
   ::nSubKeys := 0
   ::nMaxSubKeyLen := 0
   ::nMaxClassLen := 0
   ::nValues := 0
   ::nMaxValueNameLen := 0
   ::nMaxValueLen := 0
   ::nSecurityDescriptor := 0
   ::cLastWriteTime := ""
   ::nLastWriteTime := 0

   // Keep setting when doing :MoveUp / :MoveDown
   IF !lReInit
      ::cReadBinaryTypeAs := "A"                                      // Read Binary data into an array
      ::cSaveStringTypeAs := "C"                                      // Save Binary data into an array
   ENDIF

   IF !EMPTY( cRegKey )
      IF cRegKey = "\"                                                // remove backslash
         cRegKey := SUBSTR( cRegKey, 2 )
      ENDIF

      nPos := AT( "\", cRegKey )                                      // det. name of subkey

      DO CASE
         CASE cRegKey = "HKEY_LOCAL_MACHINE"
            ::nHKey := HKEY_LOCAL_MACHINE

         CASE cRegKey = "HKEY_CLASSES_ROOT"
            ::nHKey := HKEY_CLASSES_ROOT

         CASE cRegKey = "HKEY_USERS"
            ::nHKey := HKEY_USERS

         CASE cRegKey = "HKEY_CURRENT_USER"
            ::nHKey := HKEY_CURRENT_USER

         CASE cRegKey = "HKEY_CURRENT_CONFIG"
            ::nHKey := HKEY_CURRENT_CONFIG

         CASE cRegKey = "HKEY_DYN_DATA"
            ::nHKey := HKEY_DYN_DATA

         OTHERWISE
            ::nHKey := HKEY_NO_KEY

      ENDCASE

      IF ::nHKey # HKEY_NO_KEY                                        // if root key exists

         IF nPos # 0
            ::cKey := SUBSTR( cRegKey, nPos + 1 )
         ELSE
            ::cKey := ""
         ENDIF

         ::RegOpenKeyEx( ::cKey, KEY_QUERY_VALUE )                    // open key

         IF ::nCallSuccess = ERROR_SUCCESS
            ::lStatus := .T.
         ELSE
            IF lCreate
               ::RegCreateKeyEx( ::cKey )
               IF ::nCallSuccess = ERROR_SUCCESS
                  ::lStatus := .T.
               ENDIF
            ENDIF
         ENDIF

         ::RegCloseKey()

      ENDIF
   ENDIF

   SET( _SET_EXACT, lSetExact )

   RETURN self

   /*

   Registry Key erzeugen, hat nur eine Bedeutung, wenn der Key noch nicht existiert
   d.H. ::Status() == .F.

   Create mu nicht aufgerufen werden, wenn der Key existiert oder die init Methode
   mit lCreate == .T. aufgerufen wurde.

===================================

   Create registry key. Only works if the key does not exist (::Status() = .F.)

   Create is not needed if the Key exists or you call the init method with
   lCreate == .T.

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:Create()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:Create

   IF ::lStatus == .F. .AND. ::nHKey # HKEY_NO_KEY

      ::RegOpenKeyEx( ::cKey, KEY_CREATE_SUB_KEY )                    // open registry key

      IF ::nCallSuccess = ERROR_SUCCESS
         ::lStatus := .T.
      ELSE
         ::RegCreateKeyEx( ::cKey )
         IF ::nCallSuccess = ERROR_SUCCESS
            ::lStatus := .T.
         ENDIF
      ENDIF

      ::RegCloseKey()                                                 // close registry key

   ENDIF

   RETURN self

   /*
   Registry Key ffnen

================================

   Open registry key

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:RegOpenKeyEx()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:RegOpenKeyEx( cSubKey, nAccess )

LOCAL nNewKeyHandle := 0

   DEFAULT nAccess TO KEY_ALL_ACCESS

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success
   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   IF ::nhKey # HKEY_NO_KEY
      IF UseAnsi = .T.
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegOpenKeyExA", ::nhKey, ;
                                    cSubKey, 0, nAccess, @nNewKeyHandle )
      ELSE
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegOpenKeyExW", ::nhKey, ;
                                    cSubKey, 0, nAccess, @nNewKeyHandle )
      ENDIF
      IF ::nCallSuccess = ERROR_SUCCESS
         ::nKeyHandle := nNewKeyHandle
         ::RegQueryInfoKey()
      ENDIF

   ENDIF

   RETURN ::nCallSuccess

   /*
   Registry-Schlssel schlieen

=======================================

   Close registry key

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:RegCloseKey()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:RegCloseKey

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegCloseKey", ::nKeyHandle )
   IF ::nCallSuccess = ERROR_SUCCESS
      ::nKeyHandle := HKEY_NO_KEY
   ENDIF

   IF ::nDllHandle # 0
*      DllUnload( ::nDllHandle )
      ::nDllHandle := 0
   ENDIF

   RETURN ::nCallSuccess

   /*
   Neuen Registry Key erzeugen

=============================================

   Create new key

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:RegCreateKeyEx()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:RegCreateKeyEx( cSubKey )

LOCAL nNewKeyHandle := 0, nDisposition := 0
   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   IF UseAnsi = .T.
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegCreateKeyExA", ::nHKey, ;
                                 cSubKey, 0, "", REG_OPTION_NON_VOLATILE, ;
                                 KEY_ALL_ACCESS, 0, @nNewKeyHandle, @nDisposition )
   ELSE
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegCreateKeyExW", ::nHKey, ;
                                 cSubKey, 0, "", REG_OPTION_NON_VOLATILE, ;
                                 KEY_ALL_ACCESS, 0, @nNewKeyHandle, @nDisposition )
   ENDIF

   IF ::nCallSuccess = ERROR_SUCCESS
      ::nKeyHandle := nNewKeyHandle
      ::RegQueryInfoKey()
      ::RegCloseKey()
   ENDIF

   RETURN ::nCallSuccess

   /*
   Neuen Subkey unterhalb des existierenden anlegen

========================================================

   Create subkey below existing key

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:NewKey()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:NewKey( cSubKey, lKeepOld )

LOCAL cOldKey := ::KeyName(), lSuccess

   DEFAULT cSubKey TO ""
   DEFAULT lKeepOld TO .T.

   IF !EMPTY( cSubkey )
      ::Init( cOldKey + "\" + cSubKey, .T., .T. )
      IF !::lStatus
         ::Init( cOldKey,, .T. )
      ELSE
         lSuccess := .T.
         IF lKeepOld
            ::Init( cOldKey,, .T. )
         ENDIF
      ENDIF
   ENDIF

   RETURN lSuccess

   /*
   Subkey unterhalb des existierenden lschen

============================================================

   Delete subkey (including all contained values and subkeys)

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:DelKey()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:DelKey( cSubKey )

LOCAL lRet := .F.

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::RegOpenKeyEx( ::cKey, KEY_ALL_ACCESS )
   IF ::nCallSuccess = ERROR_SUCCESS
      IF UseAnsi = .T.
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegDeleteKeyA", ::nKeyHandle, ;
                                    cSubKey )
      ELSE
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegDeleteKeyW", ::nKeyHandle, ;
                                    cSubKey )
      ENDIF

      lRet := ::nCallSuccess = ERROR_SUCCESS

   ENDIF
   ::RegCloseKey()

   RETURN lRet

   /*
   Informationen ber Registry Key ermitteln und in den vorgesehenen iVars
   ablegen.

====================================================================================

   Gather information about registry key

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:RegQueryInfoKey()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:RegQueryInfoKey

LOCAL lpClass                := "", lpcbClass := 0, lpReserved := 0, lpcSubKeys := 0
LOCAL lpcbMaxSubKeyLen       := 0, lpcbMaxClassLen := 0
LOCAL lpcValues              := 0, lpcbMaxValueNameLen := 0, lpcbMaxValueLen := 0
LOCAL lpcbSecurityDescriptor := 0, lpftLastWriteTime := "        "

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   IF UseAnsi = .T.
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegQueryInfoKeyA", ::nKeyHandle, ;
                                 @ lpClass, ;
                                 @lpcbClass, ;
                                 lpReserved, ;
                                 @lpcSubKeys, ;
                                 @lpcbMaxSubKeyLen, ;
                                 @lpcbMaxClassLen, ;
                                 @lpcValues, ;
                                 @lpcbMaxValueNameLen, ;
                                 @lpcbMaxValueLen, ;
                                 @lpcbSecurityDescriptor, ;
                                 @lpftLastWriteTime )
   ELSE
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegQueryInfoKeyW", ::nKeyHandle, ;
                                 @ lpClass, ;
                                 @lpcbClass, ;
                                 lpReserved, ;
                                 @lpcSubKeys, ;
                                 @lpcbMaxSubKeyLen, ;
                                 @lpcbMaxClassLen, ;
                                 @lpcValues, ;
                                 @lpcbMaxValueNameLen, ;
                                 @lpcbMaxValueLen, ;
                                 @lpcbSecurityDescriptor, ;
                                 @lpftLastWriteTime )
   ENDIF

   IF ::nCallSuccess = ERROR_SUCCESS
      ::cClass := lpClass
      ::nClass := lpcbClass
      ::nSubKeys := lpcSubKeys
      ::nMaxSubKeyLen := lpcbMaxSubKeyLen
      ::nMaxClassLen := lpcbMaxClassLen
      ::nValues := lpcValues
      ::nMaxValueNameLen := lpcbMaxValueNameLen
      ::nMaxValueLen := lpcbMaxValueLen

      // the following is only valid under WIN/NT
      ::nSecurityDescriptor := lpcbSecurityDescriptor
      ::cLastWriteTime := lpftLastWriteTime
      ::nLastWriteTime := Bin2U( SUBSTR( ::cLastWriteTime, 1, 4 ) ) + ;
                                 Bin2U( SUBSTR( ::cLastWriteTime, 5, 4 ) ) * 2 ^ 32
   ENDIF

   RETURN ::nCallSuccess

   /*
   Subkeys aus der Registry enumerieren (Aufzhlen)
   Hilfsmethode, vorher mu der Registry-Key mit ::RegOpenKeyEx geffnet
   worden sein.

=================================================================================

   Enumerate subkeys, key has to be opened before with ::RegOpenKeyEX

*/

*+--------------------------------------------------------------------
*+
*+    Method xbpReg:RegEnumKeyEx()
*+
*+--------------------------------------------------------------------
*+
METHOD xbpReg:RegEnumKeyEx( nIndex )

LOCAL cBuffer           := SPACE( ::nMaxSubKeyLen + 1 )
LOCAL nBufLen           := ::nMaxSubKeyLen + 1
LOCAL lpftLastWriteTime := "        "

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   IF UseAnsi = .T.
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegEnumKeyExA", ::nKeyHandle, ;
                                 nIndex, @cBuffer, @nBufLen, 0, 0, 0, @lpftLastWriteTime )
   ELSE
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegEnumKeyExW", ::nKeyHandle, ;
                                 nIndex, @cBuffer, @nBufLen, 0, 0, 0, @lpftLastWriteTime )
   ENDIF

   IF ::nCallSuccess = ERROR_SUCCESS
      ::cLastWriteTime := lpftLastWriteTime
      ::nLastWriteTime := Bin2U( SUBSTR( ::cLastWriteTime, 1, 4 ) ) + ;
                                 Bin2U( SUBSTR( ::cLastWriteTime, 5, 4 ) ) * 2 ^ 32
   ENDIF

   RETURN TRIM( cBuffer )

   /*
   Values aus der Registry enumerieren (Aufzhlen)
   Hilfsmethode, vorher mu der Registry-Key mit ::RegOpenKeyEx geffnet
   worden sein.

=================================================================================

   Enumerate values, key has to be opened before with ::RegOpenKeyEx

*/

*+--------------------------------------------------------------------
*+
*+    Method xbpReg:RegEnumValue()
*+
*+--------------------------------------------------------------------
*+
METHOD xbpReg:RegEnumValue( nIndex, lWithValue )

LOCAL cBuffer1          := SPACE( ::nMaxValueNameLen + 1 )
LOCAL nBufLen1          := ::nMaxValueNameLen + 1
LOCAL cBuffer2          := SPACE( ::nMaxValueLen + 1 )
LOCAL nBufLen2          := ::nMaxValueLen + 1
LOCAL nType             := 0
LOCAL lpftLastWriteTime := "        "
LOCAL aRet              := {}
LOCAL bError

   DEFAULT lWithValue TO .T.

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::xValue := NIL
   IF UseAnsi = .T.
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegEnumValueA", ::nKeyHandle, ;
                                 nIndex, @cBuffer1, @nBufLen1, 0, @nType, @cBuffer2, @nBufLen2 )
   ELSE
      ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegEnumValueW", ::nKeyHandle, ;
                                 nIndex, @cBuffer1, @nBufLen1, 0, @nType, @cBuffer2, @nBufLen2 )
   ENDIF

   IF ::nCallSuccess = ERROR_SUCCESS
      AADD( aRet, TRIM( SUBSTR( cBuffer1, 1, nBufLen1 ) ) )

      IF lWithValue
         DO CASE
            CASE nType == REG_SZ
               ::xValue := SUBSTR( cBuffer2, 1, nBufLen2 - 1 )

            CASE nType == REG_DWORD
               ::xValue := BIN2L( cBuffer2 )

            CASE nType == REG_BINARY
               DO CASE
                  CASE ::cReadBinaryTypeAs = "A"
                     ::xValue := Bin2Var( cBuffer2 )

                  CASE ::cReadBinaryTypeAs = "C"
                     ::xValue := cBuffer2

                  CASE ::cReadBinaryTypeAs = "N"
                     ::xValue := BIN2L( cBuffer2 )
               ENDCASE

         ENDCASE
      ENDIF

      AADD( aRet, ::xValue )

   ENDIF

   RETURN aRet

   /*
   Wert aus Registry-Schlssel auslesen

======================================================

   Get a named value

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:GetValue()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:GetValue( cName )

LOCAL xRet  := SPACE( ::nMaxValueLen ), nRet := ::nMaxValueLen
LOCAL nType := 0
LOCAL cTemp, nPos1, nPos2

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::xValue := NIL

   ::RegOpenKeyEx( ::cKey, KEY_READ )
   IF ::nCallSuccess = ERROR_SUCCESS

      IF UseAnsi = .T.
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegQueryValueExA", ::nKeyHandle, ;
                                    @ cName, 0, @nType, @xRet, @nRet )
      ELSE
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegQueryValueExW", ::nKeyHandle, ;
                                    @ cName, 0, @nType, @xRet, @nRet )
      ENDIF

      IF ::nCallSuccess = ERROR_SUCCESS
         DO CASE
            CASE nType == REG_SZ
               ::xValue := SUBSTR( xRet, 1, nRet - 1 )

            CASE nType == REG_DWORD
               ::xValue := BIN2L( xRet )

            CASE nType == REG_BINARY
               DO CASE
                  CASE ::cReadBinaryTypeAs = "A"
                     ::xValue := Bin2Var( xRet )

                  CASE ::cReadBinaryTypeAs = "C"
                     ::xValue := xRet

                  CASE ::cReadBinaryTypeAs = "N"
                     ::xValue := BIN2L( xRet )
               ENDCASE

            CASE nType == REG_EXPAND_SZ
               ::xValue := SUBSTR( xRet, 1, nRet - 1 )
               DO WHILE "%" $ ::xValue
                  nPos1 := AtNum( "%", ::xValue, 1 )
                  nPos2 := AtNum( "%", ::xValue, 2 )
                  IF nPos2 = 0
                     EXIT
                  ENDIF
                  cTemp := SUBSTR( ::xValue, nPos1 + 1, nPos2 - nPos1 - 1 )
                  IF nPos1 == 1
                     ::xValue := GETENV( cTemp ) + SUBSTR( ::xValue, nPos2 + 1 )
                  ELSE
                     ::xValue := SUBSTR( ::xValue, 1, nPos1 - 1 ) + ;
                                         GETENV( cTemp ) + SUBSTR( ::xValue, nPos2 + 1 )
                  ENDIF
               ENDDO
         ENDCASE
      ENDIF
   ENDIF
   ::RegCloseKey()

   RETURN ::xValue

   /*
   Wert in Registry-Value ablegen, bzw. Value neu erzeugen

===================================================================

   Write/create a named value
*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:SetValue()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:SetValue( cName, xValue )

LOCAL nLen := 0, nType := REG_NONE

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   DO CASE
      CASE VALTYPE( xValue ) = "C"
         DO CASE
            CASE ::cSaveStringTypeAs = "C"                            // Save string as string (default)
               nLen := LEN( xValue ) + 1
               nType := REG_SZ

            CASE ::cSaveStringTypeAs = "B"                            // Save string as binary value
               nLen := LEN( xValue )
               nType := REG_BINARY
         ENDCASE

      CASE VALTYPE( xValue ) = "N"
         nLen := 4
         xValue := L2BIN( xValue )
         nType := REG_DWORD

      CASE VALTYPE( xValue ) = "A"
         xValue := Var2Bin( xValue )
         nLen := LEN( xValue )
         nType := REG_BINARY

   ENDCASE

   ::RegOpenKeyEx( ::cKey, KEY_WRITE + KEY_QUERY_VALUE )
   IF ::nCallSuccess = ERROR_SUCCESS
      IF UseAnsi = .T.
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegSetValueExA", ::nKeyHandle, ;
                                    @ cName, 0, nType, @xValue, nLen )
      ELSE
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegSetValueExW", ::nKeyHandle, ;
                                    @ cName, 0, nType, @xValue, nLen )
      ENDIF

      IF ::nCallSuccess = ERROR_SUCCESS
      ENDIF
   ENDIF
   ::RegCloseKey()

   RETURN ::nCallSuccess

   /*
   Value lschen

========================================

   Delete value

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:DelValue()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:DelValue( cName )

LOCAL lRet := .F.

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::RegOpenKeyEx( ::cKey, KEY_ALL_ACCESS )
   IF ::nCallSuccess = ERROR_SUCCESS

      IF UseAnsi = .T.
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegDeleteValueA", ::nKeyHandle, ;
                                    @ cName )
      ELSE
         ::nCallSuccess := DllCall("ADVAPI32.DLL", DLL_OSAPI, "RegDeleteValueW", ::nKeyHandle, ;
                                    @ cName )
      ENDIF

      lRet := ::nCallSuccess = ERROR_SUCCESS

   ENDIF
   ::RegCloseKey()

   RETURN lRet

   /*
   "Standard"-Wert des Registry-Keys abfragen oder setzen, als Beispiel siehe
   \HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\TimeZones

==================================================================================

   Get/Set "standard" value of the registry key for example see
   \HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\TimeZones

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:Standard()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:Standard( xValue )

   IF xValue # NIL
      ::SetValue( "", xValue )
   ENDIF

   RETURN ::GetValue( "" )

   /*
   ACCESS/ASSIGN stellt den xBase++ Datentyp ein, der beim Lesen von binren Daten
   aus der Registry verwendet werden soll. Standard ist "A" = Array, d.H.
   Daten mit dem Attribut REG_BINARY werden mit BIN2VAR in ein Array umgewandelt.

   Erlaubte Werte :   "C" = bernahme in eine String-Variable
                      "A" = bernahme in ein Array via BIN2VAR()
                      "N" = bernahme in eine numerische Variable via BIN2L()

   (!!! Arrays werden generell mit VAR2BIN gespeichert !!!)
==================================================================================

   ACCESS/ASSIGN to get/set the Xbase++ datatype that should be used when reading
   binary data from the registry. The standard value is "A" so that data with the
   REG_BINARY type attribute gets transferred into an array via BIN2VAR.

   Allowed values are "C" = retrieve as string
                      "A" = retrieve into an array via BIN2VAR()
                      "N" = retrieve into a numeric variable via BIN2L()

   (Arrays are always stored as REG_BINARY into the registry via VAR2BIN when
    using ::SetValue())

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:ReadBinType()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:ReadBinType( xValue )

   IF xValue # NIL
      IF VALTYPE( xValue ) = "C"
         ::cReadBinaryTypeAs := xValue
      ENDIF
   ENDIF

   RETURN ::cReadBinaryTypeAs

   /*
   ACCESS/ASSIGN stellt den Registry-Datentyp ein, der beim Schreiben von String-
   Daten in die Registry verwendet werden soll. Standard ist REG_SZ = String

   Erlaubte Werte :   "C" = Speichern als REG_SZ     = String
                      "B" = Speichern als REG_BINARY = Binr-Wert

   (!!! Arrays werden generell als REG_BINARY mit VAR2BIN gespeichert !!!)
==================================================================================

   ACCESS/ASSIGN to get/set the registry datatype that should be used when writing
   string type Xbase++ data to the registry. The standard value is "C" so strings will
   be saved as REG_SZ = zero terminated strings

   Allowed values are "C" = save as REG_SZ
                      "B" = save as REG_BINARY

   (Arrays are always stored into the registry as REG_BINARY via VAR2BIN when using
    ::SetValue())

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:SaveStringType()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:SaveStringType( xValue )

   IF xValue # NIL
      IF VALTYPE( xValue ) = "C"
         ::cSaveStringTypeAs := xValue
      ENDIF
   ENDIF

   RETURN ::cSaveStringTypeAs

   /*
   Status des Objekts abfragen, .F. wenn der Key
   noch nicht existiert

================================================

   Get object status, .F. if key does not exist

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:Status()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:Status

   RETURN ::lStatus

   /*
   Vollstndigen Namen des Registry-Keys zurckliefern

====================================================================

   Return complete registry key "path"

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:KeyName()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:KeyName

LOCAL cRet := "\"

   DO CASE
      CASE ::nHKey = HKEY_LOCAL_MACHINE
         cRet += "HKEY_LOCAL_MACHINE"

      CASE ::nHKey = HKEY_CLASSES_ROOT
         cRet += "HKEY_CLASSES_ROOT"

      CASE ::nHKey = HKEY_USERS
         cRet += "HKEY_USERS"

      CASE ::nHKey = HKEY_CURRENT_USER
         cRet += "HKEY_CURRENT_USER"

   ENDCASE

   cRet += IIF( !EMPTY( ::cKey ), "\" + ::cKey, "" )

   RETURN cRet

   /*
   Liste aller Subkeys als Array zurckgeben
   Verwendet ::RegEnumKeyEx()

==================================================

   List all subkeys (returns an array)
   Uses ::RegEnumKeyEx()

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:KeyList()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:KeyList

LOCAL nIndex, aRet := {}, cEnumKey

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::RegOpenKeyEx( ::cKey, KEY_READ )                                 // Key ffnen, Fehlercode steht in ::nCallSuccess
   IF ::nCallSuccess = ERROR_SUCCESS

      nIndex := ::nSubKeys

      DO WHILE nIndex >= 0

         IF !EMPTY( cEnumKey := ::RegEnumKeyEx( nIndex ) )
            AADD( aRet, cEnumKey )
         ENDIF

         nIndex --
      ENDDO

   ENDIF
   ::RegCloseKey()

   IF LEN( aRet ) # 0
      aRet := ASORT( aRet,,, { | c1, c2 | UPPER( c1 ) < UPPER( c2 ) } )
   ENDIF

   RETURN aRet

   /*
   Liste aller Values des Keys als 2-dimensionales Array zurckgeben
   Wird lWithValues auf .F. gesetzt ist das zweite Arrayelement jeweils NIL,
   es werden also nur die ValueNamen, nit jedoch der Wert selbst ermittelt.
   Vorgabewert ist .T. = Werte ermitteln

   Verwendet ::RegEnumValue()

==================================================

   List all values (returns an 2-dimensional array)
   Set lWithValues to .F. if you only need the value names, not the value itself
   Default is .T.

   Uses ::RegEnumValue()

*/

*+--------------------------------------------------------------------
*+
*+    Method XbpReg:ValueList()
*+
*+--------------------------------------------------------------------
*+
METHOD XbpReg:ValueList( lWithValues )

LOCAL nIndex, aRet := {}, aEnumVal

   DEFAULT lWithValues TO .T.

   ::nCallSuccess := 1                                                // 1 = no success; 0 = success

   IF ::nDllHandle = 0
*      ::nDllHandle := DllLoad( "ADVAPI32.DLL" )
   ENDIF

   ::RegOpenKeyEx( ::cKey, KEY_READ )                                 // Key ffnen, Fehlercode steht in ::nCallSuccess
   IF ::nCallSuccess = ERROR_SUCCESS

      nIndex := ::nValues
      IF nIndex # 0
         ::RegEnumValue( 0 )
      ENDIF

      DO WHILE nIndex >= 0

         IF LEN( aEnumVal := ::RegEnumValue( nIndex, lWithValues ) ) # 0
            AADD( aRet, aEnumVal )
         ENDIF

         nIndex --
      ENDDO

   ENDIF
   ::RegCloseKey()

   IF LEN( aRet ) # 0
      aRet := ASORT( aRet,,, { | c1, c2 | UPPER( c1[ 1 ] ) < UPPER( c2[ 1 ] ) } )
   ENDIF

   RETURN aRet

   /*
   Eine Ebene hher in der Schlsselhierarchie
   (hnlich dem DOS-Kommando CD..)

=====================================================

   Move one level up in the key hierarchy
   (similar to the dos command CD..)

*/

*+--------------------------------------------------------------------
*+
*+    Method xbpReg:MoveUp()
*+
*+--------------------------------------------------------------------
*+
METHOD xbpReg:MoveUp

LOCAL cNewKey, nPos, lSuccess := .F.

   nPos := RAT( "\", ::KeyName() )

   IF nPos > 1
      cNewKey := LEFT( ::KeyName, nPos - 1 )
      ::Init( cNewKey,, .T. )
      lSuccess := ::lStatus
   ENDIF

   RETURN lSuccess

   /*
   Eine Ebene tiefer in der Schlsselhierarchie

===========================================================

   Move down one level

*/

*+--------------------------------------------------------------------
*+
*+    Method xbpReg:MoveDown()
*+
*+--------------------------------------------------------------------
*+
METHOD xbpReg:MoveDown( cSubkey )

LOCAL cOldKey := ::KeyName(), lSuccess := .F.

   DEFAULT cSubKey TO ""
   IF !EMPTY( cSubkey )
      ::Init( cOldKey + "\" + cSubKey,, .T. )
      IF !::lStatus
         ::Init( cOldKey,, .T. )
      ELSE
         lSuccess := .T.
      ENDIF
   ENDIF

RETURN lSuccess

*+--------------------------------------------------------------------
*+
*+    Static Function ATNUM()
*+
*+    Called from ( regclass.prg )   2 - method xbpreg:getvalue()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION ATNUM( cSearch, cString, nCount )

LOCAL nPos, nCount2, nRet := - 1

   IF PCOUNT() <= 3 .AND. nCount >= 0

      nPos := 1
      nCount2 := 0

      DO WHILE .T.
         nRet := AT( cSearch, SUBSTR( cString, nPos ) )
         IF nRet # 0
            nCount2 ++
            IF nCount2 == nCount
               nRet += nPos - 1
               EXIT
            ELSE
               nPos += nRet
            ENDIF
         ELSE
            EXIT
         ENDIF
      ENDDO

   ENDIF

RETURN nRet

*+ EOF: REGCLASS.PRG